//
//  Functions.h
//  UsenetEngine
//
//  Created by Ger Teunis on 12/7/09.
//  Copyright (c) 2013 VortexApps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <UsenetEngine/Obfuscation.h>

@class NZB;
@class File;

@interface Functions : NSObject {

}

+(NSString *)stringByIterativelyResolvingSymlinkOrAlias:(NSString*)path;
+(bool)string:(NSString*)input matchedFilterSet:(NSArray*)filterSet fullKeywordMatch:(bool)fullMatch;
+(NSString*)generateUUID;
+(NSString*)generateApiKey;
+(bool)pathIsAlias:(NSString*)path;
+(bool)fileBusy:(NSString*)file;
+(NSString*)getValidFilename:(NSString*)fileName;
+(NSUInteger)IndexOfString:(NSData*)data Needle:(NSString*)needle Skip:(NSUInteger)skip;
+(NSUInteger)IndexOfStringReverse:(NSData*)data Needle:(NSString*)needle Skip:(NSUInteger)skip;
+(bool)StringMatchesRegEx:(NSString*)data RegEx:(NSString*)Regex;
+(NSRange)rangeRexEx:(NSString*)strSearch RegEx:(NSString*)RegEx;
+(NSArray *)componentsMatched:(NSString*)strSearch RegEx:(NSString *)regex;
+(NSString*)stringByMatching:(NSString*)strSearch RegEx:(NSString*)regex groupIndex:(int)groupIndex;
+(NSArray*)arrayOfCaptureComponentsMatchedByRegex:(NSString*)strSearch RegEx:(NSString*)regex;
+(NSString*)replaceOccurrencesOfRegex:(NSString*)data regEx:(NSString*)replace withString:(NSString*)with;
+(NSString*)valueForAttribute:(NSString*)haystack Attribute:(NSString*)attributeName StopOnSpace:(bool)StopOnSpace;
+(NSString*)NNTPCodeToString:(int)nntpCode;
+(NSString*)titleFromNZBFileName:(NSString*)fileName;
+(NSString*)copyDownloadPath:(NSString*)nzbFilename DoNotAsk:(bool)doNotAsk;
+(NSString*)getFileNameWithoutExtension:(NSString*)fileName;
+(bool)deleteFile:(NSString*)fileName tryMoveToTrash:(bool)moveToTrash;
+(NSString*)getApplicationSupportDirectory;
+(NSString*)volumeNameForPath:(NSString *)inPath;
+(long long)getFreeDiskSpaceForPath:(NSString*)Path;
+(long long)fileSize:(NSString*)filePath;
+(long long)getDiskSpaceUsedForPath:(NSString*)Path;
+(NSDate*)getPathChangeDate:(NSString*)Path;
+(NSString *)base64EncodedString:(NSData*)data;
+(NSData *)dataByDase64DecodingString:(NSString *)decode;
+(NSString*)temporaryDirectory:(BOOL)createIfRequired;
+(bool)getSystemVersionMajor:(unsigned *)major minor:(unsigned *)minor bugFix:(unsigned *)bugFix;
+(NSString*)getNZBStatusString:(NZB*)nzbFile;
+(NSString*)getFileStatusString:(File*)file;
+(NSString*)speedBytesToString:(unsigned long long)speedInBytes;
+(NSString*)sizeToString:(unsigned long long)left SecondSizeOrZero:(unsigned long long)right Minimal:(bool)minimal;
+(NSString*)secondsToString:(int)time Compact:(bool)compact VeryCompact:(bool)veryCompact;
+(int)compareVersion:(NSString*)leftVersion ToVersion:(NSString*)rightVersion;
+(bool)httpDownloadFile:(NSString*)url
               withData:(NSMutableData*)data
            outFileName:(NSMutableString*)fileName
             outHeaders:(NSMutableDictionary*)outHeaders;
+(bool)httpDownloadFile:(NSString*)url
                 method:(NSString*)method
            requestBody:(NSData*)body
         requestHeaders:(NSDictionary*)headers
               withData:(NSMutableData*)data
            outFileName:(NSMutableString*)fileName
             outHeaders:(NSMutableDictionary*)outHeaders;
+(NSData *)gzipInflate:(NSData*)inData;
+(NSURL*)defaultDownloadURL;
+(NSURL*)defaultCompletedURL;
+(NSURL*)defaultNZBWatchURL;
+(int)getPar2BlocksViaFilename:(NSString*)fileName;
+(NSString*)getApplicationName;
+(NSDate*)dateFromYear:(int)year Month:(int)month Day:(int)day Hour:(int)hour Minute:(int)minute Second:(int)second;
+(bool)createDirectoryRespectingVolume:(NSString*)directory;

#pragma mark -
#pragma mark Non sandboxable functions
+(bool)makeFrontProcessNamed:(NSString*)appName;
+(bool)SendAppleEvent:(AEEventID)event ToAppsNamed:(NSString*)appName;
+(OSStatus)SendAppleEventToSystemProcess:(AEEventID)EventToSend;
+(bool)preventSystemSleep;
+(bool)allowSystemSleep;
+(bool)preventAppNap;
+(bool)allowAppNap;
+(bool)detectVLCAvailable;
+(bool)launchVLCPreview:(NSString*)fileName;
+(NSString*)mountVolumeWithMountURL:(NSString*)mountUrl;
+(NSString*)getVolumeMountURL:(NSString*)volumeName;

+(BOOL)checkBundleSigniture:(NSString*)title forBundle:(NSString*)bundle;

#pragma mark -
#pragma mark Keychain code
+(NSString*)serverPassword;
+(bool)setServerPassword:(NSString*)newPassword;
+(OSStatus)keyChainWithPassword:(NSString**)outPassword AndItemRef:(SecKeychainItemRef *)itemRef;
+(OSStatus)updateServerPassword:(NSString*)newPassword ItemRef:(SecKeychainItemRef*)itemRef;

@end
