//
//  ServerHandler.h
//  UsenetEngine
//
//  Created by Ger Teunis on 11/19/10.
//  Copyright (c) 2013 VortexApps. All rights reserved.
//

#import <Foundation/Foundation.h>
@class ConnectionPoolItem;
@class NZBqueue;

@interface ServerHandler : NSObject <NSCoding, NSCopying> {
    NZBqueue *nzbQueue;
    
    bool enabled;
    NSString* serverName;
    NSString* serverURL;
    NSString* serverUserName;
    NSString* serverPassword;
    NSNumber *serverPort;
    NSNumber *serverConnections;
    bool serverUseSSL;
    
    NSMutableArray *connectionPool;
    NSLock *connectionQueueLock;
	int previousGlobalConnectResultCode;
	NSDate *previousGlobalConnectResultCodeDate;
    NSDate *previousGlovalConnectionResultCheckDate;
    bool stopPool;
    
    bool isPrimaryServer;
    bool cantConnectToServer;
    NSNumber *retentionDays;
    
    bool volumeLimitedServer;
    bool volumeResetEveryMonth;
    int volumeLastResetMonth;
    bool volumeLimitWarningShown;
    int volumeLimit;
    long long volumeDownloaded;
    ServerHandler *originalServerHandler;
    
    NSString *uniqueServerIdentifier;
}

@property (nonatomic) bool enabled;
@property (nonatomic, retain) NZBqueue* nzbQueue;
@property (nonatomic, retain) NSString* serverName;
@property (nonatomic, retain) NSString* serverURL;
@property (nonatomic, retain) NSString* serverUserName;
@property (nonatomic, retain) NSString* serverPassword;
@property (nonatomic, retain) NSNumber *serverPort;
@property (nonatomic, retain) NSNumber *serverConnections;
@property (nonatomic) bool serverUseSSL;
@property (nonatomic, retain) NSLock *connectionQueueLock;
@property (nonatomic) int previousGlobalConnectResultCode;
@property (nonatomic, retain) NSDate *previousGlobalConnectResultCodeDate;
@property (nonatomic) bool stopPool;
@property (nonatomic) bool isPrimaryServer;
@property (nonatomic) bool cantConnectToServer;
@property (nonatomic, retain) NSNumber *retentionDays;
@property (nonatomic) bool volumeLimitedServer;
@property (nonatomic) bool volumeResetEveryMonth;
@property (nonatomic) int volumeLastResetMonth;
@property (nonatomic) bool volumeLimitWarningShown;
@property (nonatomic) int volumeLimit;
@property (nonatomic) long long volumeDownloaded;
@property (nonatomic, retain) ServerHandler *originalServerHandler;

#pragma mark -
#pragma mark Connection methods
-(BOOL)hasDownloadingConnection;
-(void)resetConnections;
-(void)createConnectionPoolItems;
-(void)removeConnection:(ConnectionPoolItem*)connection;
-(void)detectConnectResultErrors;
-(int)getNumberOfDownloadingConnections;

#pragma mark -
#pragma mark Rest
-(int)openConnections;
-(int)getBytesTransferred;
-(void)setNewThrottleDelay:(float)delay;
-(void)setNewNetworkReadSize:(int)size;
-(NSString*)uniqueServerIdentifier;
-(void)addTransferredBytes:(int)bytes;
-(void)setTransferredBytes:(long long)bytes;
-(bool)testSettings:(NSString**)loginResult;
-initWithQueue:(NZBqueue*)queue
       Enabled:(bool)isEnabled
    ServerName:(NSString*)ServerName
     ServerURL:(NSString*)ServerURL
      UserName:(NSString*)UserName
      Password:(NSString*)Password
          Port:(NSNumber*)portNumber
        UseSSL:(bool)useSSL
ConnectionNumber:(NSNumber*)connectionNumber
IsPrimaryServer:(bool)IsPrimaryServer
 RetentionDays:(NSNumber*)RetentionDays
VolumeLimitedServer:(bool)VolumeLimitedServer
VolumeResetEveryMonth:(bool)VolumeResetEveryMonth
VolumeLastResetMonth:(int)VolumeLastResetMonth
VolumeLimitWarningShown:(bool)VolumeLimitWarningShown
   VolumeLimit:(int)VolumeLimit
VolumeDownloaded:(long long)VolumeDownloaded;

@end
